###################################################
### chunk number 1: preliminaries
###################################################
#line 85 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
library(ReacTran)
options(prompt = " ")


###################################################
### chunk number 2: 
###################################################
#line 178 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
Grid <- setup.grid.1D(N = 1000, L = 10)


###################################################
### chunk number 3: 
###################################################
#line 185 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
r  <- setup.prop.1D(grid = Grid, func = function(r) r)
r2 <- setup.prop.1D(grid = Grid, func = function(r) r^2)


###################################################
### chunk number 4: 
###################################################
#line 197 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
library(ReacTran)
pde1D <-function(t, C, parms, A = 1)  {
  tran   <- tran.1D (C = C, A = A, D = D, C.down = Cext, dx = Grid)$dC
  list(tran - Q)  # the return value: rate of change
}


###################################################
### chunk number 5: 
###################################################
#line 207 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
D    <- 1    # diffusion constant
Q    <- 1    # uptake rate
Cext <- 20


###################################################
### chunk number 6: 
###################################################
#line 235 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
library(rootSolve)
Cartesian   <- steady.1D(y = runif(Grid$N),
  func = pde1D, parms = NULL, nspec = 1, A = 1)
Cylindrical <- steady.1D(y = runif(Grid$N),
  func = pde1D, parms = NULL, nspec = 1, A = r)


###################################################
### chunk number 7: 
###################################################
#line 242 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
print(system.time(
  Spherical   <- steady.1D(y = runif(Grid$N),
    func = pde1D, parms = NULL, nspec = 1, A = r2)
))


###################################################
### chunk number 8: pde
###################################################
#line 254 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
plot(Cartesian, Cylindrical, Spherical, grid = Grid$x.mid, 
   main = "steady-state PDE", xlab = "x", ylab = "C",
   col = c("darkgreen", "blue", "red"), lwd = 3, lty = 1:3)

legend("bottomright", c("cartesian", "cylindrical", "spherical"),
  col = c("darkgreen", "blue", "red"), lwd = 3, lty = 1:3)


###################################################
### chunk number 9: figpde
###################################################
#line 266 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
#line 254 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw#from line#266#"
plot(Cartesian, Cylindrical, Spherical, grid = Grid$x.mid, 
   main = "steady-state PDE", xlab = "x", ylab = "C",
   col = c("darkgreen", "blue", "red"), lwd = 3, lty = 1:3)

legend("bottomright", c("cartesian", "cylindrical", "spherical"),
  col = c("darkgreen", "blue", "red"), lwd = 3, lty = 1:3)
#line 267 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"


###################################################
### chunk number 10: 
###################################################
#line 278 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
max(abs(Q/6/D*(r2$mid - 10^2) + Cext - Spherical$y))
max(abs(Q/4/D*(r2$mid - 10^2) + Cext - Cylindrical$y))
max(abs(Q/2/D*(r2$mid - 10^2) + Cext - Cartesian$y))


###################################################
### chunk number 11: 
###################################################
#line 288 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
require(deSolve)
times <- seq(0, 100, by = 1)
system.time(
  out <- ode.1D(y = rep(1, Grid$N), times = times, func = pde1D,
    parms = NULL, nspec = 1, A = r2)
)


###################################################
### chunk number 12: 
###################################################
#line 301 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
tail(out[, 1:4], n = 3)


###################################################
### chunk number 13: yy
###################################################
#line 308 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
image(out, grid = Grid$x.mid, xlab = "time, days", 
     ylab = "Distance, cm", main = "PDE", add.contour = TRUE)


###################################################
### chunk number 14: 
###################################################
#line 359 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
dx    <- 0.2
xgrid <- setup.grid.1D(-100, 100, dx.1 = dx)
x     <- xgrid$x.mid
N     <- xgrid$N


###################################################
### chunk number 15: 
###################################################
#line 366 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
uini  <- exp(-0.05 * x^2)
vini  <- rep(0, N)
yini  <- c(uini, vini)
times <- seq (from = 0, to = 50, by = 1)


###################################################
### chunk number 16: 
###################################################
#line 377 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
wave <- function (t, y, parms) {
  u1 <- y[1:N]
  u2 <- y[-(1:N)]

  du1 <- u2
  du2 <- tran.1D(C = u1, C.up = 0, C.down = 0, D = 1, dx = xgrid)$dC
  return(list(c(du1, du2)))
}


###################################################
### chunk number 17: 
###################################################
#line 390 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
out <- ode.1D(func = wave, y = yini, times = times, parms = NULL,
         nspec = 2, method = "ode45", dimens = N, names = c("u", "v"))


###################################################
### chunk number 18: wave
###################################################
#line 400 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
matplot.1D(out, which = "u", subset = time %in% seq(0, 50, by = 10), 
   type = "l", col = c("black", rep("darkgrey", 5)), lwd = 2,
   grid = x, xlim = c(-50,50))
legend("topright", lty = 1:6, lwd = 2, col = c("black", rep("darkgrey", 5)), 
       legend = paste("t = ",seq(0, 50, by = 10)))


###################################################
### chunk number 19: wave
###################################################
#line 410 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
#line 400 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw#from line#410#"
matplot.1D(out, which = "u", subset = time %in% seq(0, 50, by = 10), 
   type = "l", col = c("black", rep("darkgrey", 5)), lwd = 2,
   grid = x, xlim = c(-50,50))
legend("topright", lty = 1:6, lwd = 2, col = c("black", rep("darkgrey", 5)), 
       legend = paste("t = ",seq(0, 50, by = 10)))
#line 411 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"


###################################################
### chunk number 20: waveimg
###################################################
#line 421 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
par(mar=c(0,0,0,0))
image(out, which = "u", method = "persp", main = "",
        border = NA, col = "lightblue", box = FALSE, 
        shade = 0.5, theta = 0, phi = 60)                                                     


###################################################
### chunk number 21: 
###################################################
#line 468 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
require(ReacTran)
pde2D <- function (t, y, parms) {
  CONC <- matrix(nr = n, nc = n, y)
  Tran <- tran.2D(CONC, D.x = Dx, D.y = Dy, dx = dx,  dy = dy)
  dCONC <- Tran$dC - r * CONC
  dCONC[ii]<- dCONC[ii] + p
  return(list(as.vector(dCONC)))
}


###################################################
### chunk number 22: 
###################################################
#line 485 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
n  <- 100
dy <- dx <- 1
Dy <- Dx <- 2
r  <- 0.001
p  <- runif(50)
ii <- trunc(cbind(runif(50)*n, runif(50)*n) + 1)


###################################################
### chunk number 23: 
###################################################
#line 499 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
require(rootSolve)
Conc0 <- matrix(nr = n, nc = n, 10.)
print(system.time(
  ST <- steady.2D(y = Conc0, func = pde2D, parms = NULL, dimens = c(n, n),
   lrw = 600000)
))


###################################################
### chunk number 24: D2
###################################################
#line 507 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
image(ST, main = "steady-state 2-D PDE")


###################################################
### chunk number 25: D2fig
###################################################
#line 513 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"
#line 507 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw#from line#513#"
image(ST, main = "steady-state 2-D PDE")
#line 514 "d:/Rcompile/CRANpkg/local/2.12/ReacTran/inst/doc/PDE.Rnw"


